/*  JarWriter.java
 *
 *  Copyright (C) 2002-2003 Dominik Werthmueller
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
 
package ch.fulgur.jarbuilder;

import java.util.jar.JarOutputStream;
import java.util.zip.ZipEntry;
import java.io.*;
import javax.swing.SwingUtilities;
 
 
 
/** 
 * This class allows to easily build a jar file.
 *
 * @author Dominik Werthmueller
 * 
 */
 
public class JarWriter {
	
	private String[] files;
	private String manifest;
	private File manifestFile;
	private boolean includeManifest = true;
	private boolean loadManifest = false;
	private boolean aborted = false;
	private int writtenBytes;
	private int totalFileSize;
	
	private final int BUFFERSIZE = 32768;
	
	private Panel4 panel4;
	
	
	// ---------- Begin Method Section ----------
	
	


	/**
	 * set the aborted variable
	 *
	 * @param true if the creation process should be stopped
	 */
	public void setAborted(boolean b) {
		aborted = b;
	}
	
	
	/** 
	 * set the manifest include option
	 *
	 * @param true for including manifest, false for non-including manifest
	 */
	public void setIncludeManifest(boolean b) {
		includeManifest = b;
	}
	
	
	/** 
	 * return the manifest include option
	 *
	 * @return manifest include option
	 */
	public boolean getIncludeManifest() {
		return includeManifest;
	}
	
	
	/** 
	 * set the manifest load option
	 *
	 * @param true for loading manifest, false for non-loading manifest
	 */
	public void setLoadManifest(boolean b) {
		loadManifest = b;
	}
	
	
	/** 
	 * return the manifest load option
	 *
	 * @return manifest include option
	 */
	public boolean getLoadManifest() {
		return loadManifest;
	}
	
		
	/** 
	 * set the files for the jar file
	 *
	 * @param files for the jar file
	 */
	public void setFiles(String[] f) {
		files = f;
	}
	
	
	/** 
	 * return the files of the jar file
	 *
	 * @return files of the jar file
	 */
	public String[] getFiles() {
		return files;
	}
	
	
	/** 
	 * return the file at the given index of the jar file
	 *
	 * @param index
	 * @return the file at the given index of the jar file
	 */
	public String getFile(int i) {
		return files[i];
	}
	
	
	/** 
	 * set the manifest content for the jar file
	 *
	 * @param manfest content for the jar file
	 */
	public void setManifest(String m) {
		manifest = m;
	}
	
	
	/** 
	 * return the manifest content of the jar file
	 *
	 * @return manifest content of the jar file
	 */
	public String getManifest() {
		return manifest;
	}
	
	
	/** 
	 * set the manifest file for the jar file
	 *
	 * @param manfest file for the jar file
	 */
	public void setManifestFile(File f) {
		manifestFile = f;
	}
	
	
	/** 
	 * return the manifest file of the jar file
	 *
	 * @return manifest file of the jar file
	 */
	public File getManifestFile() {
		return manifestFile;
	}
	
	
	
	/** 
	 * return the number of files of the jar file
	 *
	 * @return number of files of the jar file
	 */
	public int getLength() {
		return files.length;
	}
	
	
	/** 
	 * write the manifest file
	 *
	 * @return the manifest file
	 */
	private File writeManifest() {
		File fManifest = new File("Manifest.tmp");
		try {
			BufferedWriter bout = new BufferedWriter(new FileWriter(fManifest));
		
			if (!manifest.equals("")) {
				bout.write("Manifest-Version: 1.0");
				bout.newLine();
				bout.write("Created-By: JarBuilder 0.8.0");
				bout.newLine();
				bout.write(manifest);
				bout.newLine();
				bout.newLine();
			}
			
			bout.close();
		} catch (Exception e) {
			System.out.println("[writeManifest(), JarWriter] ERROR\n" + e);
		}
		
		return fManifest;	
	}
	
	
	/**
	 * set the message during the creation process
	 *
	 * @param value to set
	 */
	private void setMessage(int a) {
		String s = new String(new Integer(a).toString());
		int l = s.length();
			
		if (l > 3 && l < 6) s = s.substring(0, l - 3) + "'" + s.substring(l - 3, l);
		if (l == 6) s = s.substring(0, l - 3) + "'" + s.substring(l - 3, l);
		if (l > 6) s = s.substring(0, l - 6) + "'" + s.substring(l - 6, l - 3) + "'" + s.substring(l - 3, l);
		
		final String s2 = s;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				panel4.setMessageText("Compressing " + s2 + " KB - please wait...");
			}
		});
	}

	
	/** 
	 * set a new value to the progress bar in JarBuilder
	 *
	 * @param value to set
	 * @param String to set
	 */
	private void setBar(final int s, final String st) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				panel4.setBarValue(s);
				panel4.setBarText(st);
			}
		});
	}
	

	/**
	 * set the maximum value of the progress bar in JarBuilder
	 *
	 * @param maximum value to set
	 */
	private void setBarMax(final int m) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				panel4.setBarMaximum(m);
			}
		});	
	}
	
	
	/**
	 * calculate the size of one file (kb)
	 *
	 * @param file
	 */
	private int getFileSize(File f) {
		File[] dContent;
		int dSize;
		
		if (f.isDirectory() ==  false) {
			return (int)(f.length() / 1024f);
		}
		else {
			dContent = f.listFiles();
			dSize = 0;
			for (int a = 0; a < dContent.length; a++) {
				dSize += getFileSize(dContent[a]);	
			}
			return dSize;
		}
	}

	
	/**
	 * calculate the size of all files (kb)
	 *
	 * @return the total file size
	 */
	private int getTotalFileSize() {
		File f;
		int totalSize = 0;
		
		for (int i = 0; i < files.length; i++) {
			f = new File(files[i]);
			totalSize += getFileSize(f);		
		}
		return totalSize;
	}
	
	 
	/** 
	 * write the manifest entry in the jar file
	 *
	 * @param file to write
	 * @param current JarOutputStream
	 */
	private void writeManifestEntry(File f, JarOutputStream out) {
		// buffer
		byte[] buffer = new byte[BUFFERSIZE];
			
		// read bytes
		int bytes_read;
		
		try {
			BufferedInputStream in = new BufferedInputStream(new FileInputStream(f), BUFFERSIZE);
			String en = "META-INF" + "/" + "MANIFEST.MF";
			out.putNextEntry(new ZipEntry(en));
			while ((bytes_read = in.read(buffer)) != -1) {
				out.write(buffer, 0, bytes_read);
			}
				
			in.close();
			out.closeEntry();
		} catch (Exception e) {
			System.out.println("[writeManifestEntry(), JarWriter] ERROR\n" + e);
		}
	}


	/** 
	 * write entries in the jar file
	 *
	 * @param file to write
	 * @param current JarOutputStream
	 * @param depth of the directory structure
	 * @return true if the file was successfully written, false if there was an error during the writing
	 */
	private boolean writeEntry(File f, JarOutputStream out, int depth) {
		String en = "";
		File[] dContent;
		int i;
		String fPath;
		
		// buffer
		byte[] buffer = new byte[BUFFERSIZE];
			
		// read bytes
		int bytes_read;
		
		try {
			if (f.isDirectory() == false) {	
				BufferedInputStream in = new BufferedInputStream(new FileInputStream(f), BUFFERSIZE);
				
				i = f.getPath().length();
				fPath = f.getPath();
				for (int a = 0; a <= depth; a++) {
					i = fPath.lastIndexOf(VarUtils.FILESEPARATOR, i) - 1;
				}
				
				
				en = fPath.substring(i + 2, fPath.length());
				out.putNextEntry(new ZipEntry(en));
				
				while ((bytes_read = in.read(buffer)) != -1) {
					// do the work
					out.write(buffer, 0, bytes_read);
					
					// check if the user has aborted the creation process
					if (aborted) {
						in.close();
						out.closeEntry();
						return false;
					}
					
					// update progress bar
					writtenBytes += bytes_read;
					setBar((int)(writtenBytes / 1024), "compressing " + resizeFilePathString(f.getPath()));
				}

				in.close();
				out.closeEntry();
				return true;
			}
			else {
				dContent = f.listFiles();
				for (int a = 0; a < dContent.length; a++) {
					writeEntry(dContent[a], out, depth + 1);
					// check if the user has aborted the creation process
					if (aborted) {
						return false;
					}
				}
			}	
		} catch (Exception e) {
			System.out.println("[writeEntry(), JarWriter] ERROR\n" + e);
			return false;
		}
		return true;
	}



	/**
	 * the maximum length of the file's path (that will be displayed) should be 30
	 * paste ... in the middle
	 *
	 * @param file path string to resize
	 * @return resized string
	 */
	private String resizeFilePathString(String s) {
		if (s.length() < 30) return s;
		else {
			String s1, s2;
			int index;

			s1 = s.substring(0, s.indexOf(VarUtils.FILESEPARATOR, 5) + 1);
			s2 = s.substring(s.lastIndexOf(VarUtils.FILESEPARATOR), s.length());

			return s1 + "..." + s2;
		}
	}



	/**
	 * create a jar file including the files and the manifest file
	 *
	 * @param the jar file
	 * @param true to compress, false to not compress
	 * @return true if the file was successfully built, false if there was an error during the building process
	 */
	public boolean createJar(File fj, int compress) {
		File f;
		boolean written;
		
		try {
			// target
			JarOutputStream out = new JarOutputStream(new FileOutputStream(fj));
			out.setComment("This file was created by JarBuilder\nCheck http://fulgur.ch.vu !");
			
			// set the compression rate
			out.setLevel(compress);
			
			// preparations
			totalFileSize = getTotalFileSize();
			setBarMax(totalFileSize);
			setMessage(totalFileSize);

			setBar(0, "compressing " + resizeFilePathString(files[0]));
			aborted = false;
			writtenBytes = 0;

			
			
			// add files
			for (int i = 0; i < files.length; i++) {
				f = new File(files[i]);
				written = writeEntry(f, out, 0);
				if (!written) {
					out.close();
					fj.delete();
					return false;
				}
			}
			
			
			// add manifest
			if (includeManifest) {
				if (loadManifest == false) {
					f = writeManifest();
					writeManifestEntry(f, out);
					f.delete();
				}
				else {
					writeManifestEntry(manifestFile, out);
				}
			}
			
			out.close();
			return true;
			
		} catch (Exception e) {
			System.out.println("[createJar(), JarWriter] ERROR\n" + e);
			return false;
		}
	}
	
	
	
	// ---------- Begin Constructor Section ----------
	
	/** 
	 * construct an empty JarWriter class
	 */
	public JarWriter(Panel4 p4) {
		panel4 = p4;
		manifest = "";
	}	
	
	
	/** 
	 * construct an JarWriter class including the given files
	 */
	public JarWriter(String[] f, Panel4 p4) {
		panel4 = p4;
		files = f;
		manifest = "";
	}
	
	
	/** 
	 * construct an JarWriter class including the given files and the given Manifest file
	 */
	public JarWriter(String[] f, String m, Panel4 p4) {
		panel4 = p4;
		files = f;
		manifest = m;
	}	
}	
